#!/usr/bin/perl -w
use CGI;

# show vtx files in browser
# originally by Gerd Knorr <kraxel@goldbach.in-berlin.de>
#
# changes by Rolf Bleher <dk7in@qsl.net>,  http://www.qsl.net/dk7in
#         V1.1 28.01.2000
#         V1.0 22.05.1999
#   sorting stations and pages, some graphics, 
#   Hyperlinks directly in the text

# config
$SPOOL="/var/spool/vtx";
$VTX2ASCII="/usr/local/bin/vtx2ascii";     # from xawtv package

# some graphics                  I put them in /usr/local/httpd/htdocs/img/
$SUBPRV  = "/img/ArRedL.gif";    # put in your favorite icons here...
$SUBNXT  = "/img/ArRedR.gif";    #   small and bigger arrows
$PAGPRV  = "/img/PageL.gif";
$PAGNXT  = "/img/PageR.gif";
$SPACE1  = "/img/Sp-30x3.gif";   # icon for spacing

#--------------------------------------------------------------------------

$cgi  = new CGI;

if ($cgi->path_info eq "" || $cgi->path_info =~ /^\/[^\/]+$/) {
    print $cgi->redirect($cgi->url . $cgi->path_info . "/");
    exit;
}
($dummy,$station,$page) = split(/\//,$cgi->path_info);

# entry page - station list
if ($station !~ /\S/) {
    opendir DIR, $SPOOL || panic("can't open dir $SPOOL: $@");
    &start_page("station list",$SPOOL);
    print "<h2>station list</h2>\n<ul>\n";
    @files = sort readdir(DIR);
    foreach (@files) {
        next if ($_ =~ /^\./);
	next unless -d "$SPOOL/$_";
	print "<li><a href=\"$_/\">$_</a>\n";
    }
    print "</ul>\n";
    &finish_page;
    exit;
}

# station dir - spage list
if ($page !~ /\S/) {
    opendir DIR, "$SPOOL/$station" ||
	panic("can't open dir $SPOOL/$station: $@");
    &start_page("$station - page list","$SPOOL/$station");
    print "<h2>$station - page list</h2>\n\n";
    print "<a href=\"../\">[station list]</a><hr noshade><ul>\n";
    @files = sort grep { /\.vtx/ } readdir(DIR);
    foreach (@files) {
	print "<li><a href=\"$_\">$_</a>\n";
    }
    print "</ul>\n";
    &finish_page;
    exit;
}

# we dont have _00 but have sub page _01, so print page _01
unless (-f "$SPOOL/$station/$page") {
    # sub-page check
    if ($page =~ s/_00.vtx/_01.vtx/ && -f "$SPOOL/$station/$page") {
	print $cgi->redirect($cgi->url . "/$station/$page");
	exit;
    }
    panic("$SPOOL/$station/$page: not found");
}

# if both _00 and _01 are available: print _01 if it is newer
$dupe = $page;
if ($dupe =~ s/_00.vtx/_01.vtx/ && -f "$SPOOL/$station/$dupe") {
    if ( -M "$SPOOL/$station/$page" > -M "$SPOOL/$station/$dupe" ) { # age
        print $cgi->redirect($cgi->url . "/$station/$dupe");
        exit;
    }
}

$page =~ /(\d\d\d)_(\d\d).vtx/;
$pg    = $1;
$subpg = $2;
$pgnam = $pg;
if ($subpg > 0) {
  $pgnam = sprintf("%03d / %d",$pg,$subpg);
}

# read
undef $/;
open VTX,"$VTX2ASCII -h $SPOOL/$station/$page |" || 
    panic("can't run $VTX2ASCII: $@");
$data = <VTX>;
close VTX;

# set up all links inside the page
while (1) {
  last if ($data !~ s{
                      (                              # $1 Text vor Nummer
                       [a-zA-Z.><:]\s+               # c 123|. 123|> 123|: 123
                        |
                       [)]\s+                        # ) 123
                        |
                       <\/a>[\/-]                    # 456-123 | 456/123
                        |
                       [^0-9 ][).>:]                 # c.123|c>123|c)123|c:123
                        |
                       \s[.>]                        # ' .123'|' >123'
                        |
                       \D-\s+                        # c- 123
                        |
                       class=c\d{1,2}>\s*            # PageNr with own color
                        |
                       \s19\d\d\s+                   # 19nn 123
                      )
                      ([1-8]\d{2})                   # $2 PageNr 100-899
                      (                              # $3 Text nach PageNr
                        [ >]                         # '123 '|'123>'
                         |
                        -\d{3}                       # 123-456
                         |
                        \/                           # 123/
                         |
                        -<                           # 123-
                         |
                        \s*<\/font                   # </font
                         |
                        <[^\/]                       # <..
                      )
                     }
                      "$1<a href=$2_00.vtx>$2<\/a>$3"gx
          )
}
# we don't need a link for our own page...
$data =~ s/<a href=($pg)_00.vtx>$pg<\/a>/$1/g;

# try to get the best color for all links
# I think there is no way to set different colors for links in style sheets !?
# so I set all links to the most used color and background
for ($col = 0; $col < 64; $col++) {     # init link color counter
  $used[$col] = 0;
}
@lines = split(/<\/font>/,$data);       # count link colors
foreach $item (@lines) {
  if ($item =~ /<font class=c(\d{1,2})>.*<a href=/) {
    $used[$1]++;
  }
}
$idx = -1;
$max = 0;
for ($col = 0; $col < 64; $col++) {     # most used color?
  if ($used[$col] gt $max) {
    $idx = $col;
    $max = $used[$col];
  }
}

if ($idx ge 0) {           # set color for links in style sheet
  $data =~ s/a:link.* \}\n//;
  $data =~ s/a:vlink.* \}\n//;
  @color = qw(black red green yellow blue magenta cyan white);
  $bg = $idx % 8;
  $fg = ($idx - $bg) / 8;
  $fg = $color[$fg];
  $bg = $color[$bg];
  $licol = "a:link { color: $fg\; background-color: $bg }\na:visited { color: $fg\; background-color: $bg }\n";
  $data =~ s/(<style type="text\/css"> <!--\n)/$1$licol/;
}

# convert Umlaute
$data =~ s//\&auml;/g;
$data =~ s//\&ouml;/g;
$data =~ s//\&uuml;/g;
$data =~ s//\&szlig;/g;
$data =~ s//\&Auml;/g;
$data =~ s//\&Ouml;/g;
$data =~ s//\&Uuml;/g;

# print
start_page("$station - $pgnam","$SPOOL/$station/$page");
print("<h3><a href=\"../\">$station</a> - <a href=\"./\">$pgnam</a>");
print("<img src=\"$SPACE1\">");
printf("<a href=\"%03d_%02d.vtx\"><img src=\"$PAGPRV\" border=0 alt=prev page></a> &nbsp; ",$pg-1,0);
if ($subpg > 0) {
    printf("<a href=\"%03d_%02d.vtx\"><img src=\"$SUBPRV\" border=0 alt=prev subpage></a> &nbsp; ",$pg,$subpg-1);
    printf("<a href=\"%03d_%02d.vtx\"><img src=\"$SUBNXT\" border=0 alt=next subpage></a> &nbsp; ",$pg,$subpg+1);
}
printf("<a href=\"%03d_%02d.vtx\"><img src=\"$PAGNXT\" border=0 alt=next page></a> &nbsp; </h3>",$pg+1,0);

print("$data");

print "<br>\n";
finish_page();
exit;

#--------------------------------------------------------------------------
# create time string

@WEEK = qw(Sun Mon Tue Wed Thu Fri Sat);
@MON  = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);

sub time2str {
    @tm    = gmtime($_[0]);
    sprintf ("%s, %02d %s %04d %02d:%02d:%02d GMT",
             $WEEK[$tm[6]],$tm[3],$MON[$tm[4]],$tm[5]+1900,$tm[2],$tm[1],$tm[0]);
}

#--------------------------------------------------------------------------
# helper functions

sub start_page {
    local($title,$file) = @_;

    if ($file ne "") {
        @inode = stat $file;
        printf("Last-modified: %s\n",time2str($inode[9]));
    }

print $cgi->header({-type    => 'text/html',
                    -expires => '1s',
                    -Refresh => '120s'});

print $cgi->start_html(-title   => "Videotext: $title",
                       -BGCOLOR => '#F0F0D0');

}

#--------------------------------------------------------------------------

#    print <<EOF;
#Content-Type: text/html
#
#<html>
#<head>
#<title>Videotext: $title</title>
#<META HTTP-EQUIV="refresh" CONTENT="120">
#</head>
#<body bgcolor=white color=black link=royalblue vlink=darkblue>
#EOF
#}

#--------------------------------------------------------------------------

sub finish_page {
    print "</body></html>\n";
}

#--------------------------------------------------------------------------

sub panic {
    local($text) = @_;

    start_page("PANIC");
    print "<h2>PANIC</h2>$text";
    &finish_page;
    exit;
}

#--------------------------------------------------------------------------

